"""Product block for :class:`office router` products."""

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.site import (
    SiteBlock,
    SiteBlockInactive,
    SiteBlockProvisioning,
)
from gso.utils.shared_enums import IPv4AddressType, IPv6AddressType, PortNumber, Vendor


class OfficeRouterBlockInactive(
    ProductBlockModel,
    lifecycle=[SubscriptionLifecycle.INITIAL],
    product_block_name="OfficeRouterBlock",
):
    """An office router that's being currently inactive. See :class:`OfficeRouterBlock`."""

    office_router_fqdn: str | None = None
    office_router_ts_port: PortNumber | None = None
    office_router_lo_ipv4_address: IPv4AddressType | None = None
    office_router_lo_ipv6_address: IPv6AddressType | None = None
    office_router_site: SiteBlockInactive | None
    vendor: Vendor | None = None


class OfficeRouterBlockProvisioning(OfficeRouterBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """An office router that's being provisioned. See :class:`RouterBlock`."""

    office_router_fqdn: str | None = None
    office_router_ts_port: PortNumber | None = None
    office_router_lo_ipv4_address: IPv4AddressType | None = None
    office_router_lo_ipv6_address: IPv6AddressType | None = None
    office_router_site: SiteBlockProvisioning | None
    vendor: Vendor | None = None


class OfficeRouterBlock(OfficeRouterBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """An office router that's currently deployed in the network."""

    #:  Office router FQDN.
    office_router_fqdn: str
    #:  The port of the terminal server that this office router is connected to. Used to offer out of band access.
    office_router_ts_port: PortNumber
    #:  The IPv4 loopback address of the office router.
    office_router_lo_ipv4_address: IPv4AddressType
    #:  The IPv6 loopback address of the office router.
    office_router_lo_ipv6_address: IPv6AddressType
    #:  The :class:`Site` that this office router resides in. Both physically and computationally.
    office_router_site: SiteBlock
    #:  The vendor of an office router. Defaults to Juniper.
    vendor: Vendor = Vendor.JUNIPER
